package com.ruoyi.project.business.service.quotation.impl;

import static com.ruoyi.project.business.enums.AuditFlowEnum.BARGAIN;
import static com.ruoyi.project.business.enums.AuditStatusEnum.BARGAIN_APPROVAL3;
import static com.ruoyi.project.business.enums.AuditStatusEnum.BARGAIN_APPROVAL4;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.QuotationUtil;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.business.domain.TbSalesDiscount;
import com.ruoyi.project.business.domain.TqPriceDetail;
import com.ruoyi.project.business.domain.TqQuotation;
import com.ruoyi.project.business.domain.TqSalespersonSetPrice;
import com.ruoyi.project.business.domain.TqSpecification;
import com.ruoyi.project.business.domain.TqSpecificationNonStandard;
import com.ruoyi.project.business.domain.flow.FlowAuditParam;
import com.ruoyi.project.business.enums.AuditStatusEnum;
import com.ruoyi.project.business.enums.DelFlagEnum;
import com.ruoyi.project.business.enums.NodeRoleEnum;
import com.ruoyi.project.business.enums.QuotationMsgEnum;
import com.ruoyi.project.business.enums.QuotationStatusEnum;
import com.ruoyi.project.business.enums.SystemCheckResultEnum;
import com.ruoyi.project.business.mapper.quotation.TqQuotationMapper;
import com.ruoyi.project.business.mapper.quotation.TqSpecificationMapper;
import com.ruoyi.project.business.service.ITbSalesDiscountService;
import com.ruoyi.project.business.service.flow.IFlowService;
import com.ruoyi.project.business.service.quotation.IQuoteApprovalService;
import com.ruoyi.project.business.service.quotation.ITqPriceDetailService;
import com.ruoyi.project.business.service.quotation.ITqSalespersonSetPriceService;
import com.ruoyi.project.business.service.quotation.ITqSpecificationNonStandardService;
import com.ruoyi.project.business.service.quotation.ITqSpecificationService;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

// 报价流程审批
@Service
@Slf4j
public class QuoteApprovalServiceImpl extends ServiceImpl<TqQuotationMapper, TqQuotation> implements
    IQuoteApprovalService {

    public static final String ONE_HUNDRED = "100";

    @Resource
    private ITqPriceDetailService priceDetailService;

    // @Resource
    // private ITqQuotationService tqQuotationService;

    @Resource
    private IFlowService flowService;

    @Resource
    private ITqSalespersonSetPriceService tqSalespersonSetPriceService;

    @Resource
    private ITqSpecificationNonStandardService tqSpecificationNonStandardService;

    @Resource
    private ITqSpecificationService tqSpecificationService;

    @Resource
    private TqSpecificationMapper tqSpecificationMapper;

    @Resource
    private ITbSalesDiscountService tbSalesDiscountService;


    @Override
    @Transactional
    public void changeQuotationStatus(FlowAuditParam auditParam) {
        TqQuotation quotation = this.getById(auditParam.getBusinessId());
        if (AuditStatusEnum.INIT.getValue().equals(auditParam.getCurrentStatus())) {
            // 初始化审批流时,发邮件/站内信
            QuotationUtil.sendMsgAsync(QuotationMsgEnum.REVIEW, auditParam.getNextAuditUsers(),
                quotation.getQuotationNo());
            log.info("[{}] 未知节点状态 [{}]", auditParam.getBusinessId(), auditParam.getCurrentStatus());
            return;
        }
        String quotationStatus = null;
        JSONObject reviewResultJson = Optional.ofNullable(auditParam.getBusinessObj())
            .map(a -> a.getJSONObject("reviewResultJson")).orElse(new JSONObject());
        // 报价当前状态
        AuditStatusEnum auditStatusEnum = AuditStatusEnum.from(auditParam.getCurrentStatus());

        // 销售确认
        if (auditStatusEnum.getValue().equals(AuditStatusEnum.SALES_CONFIRM.getValue())) {
            QuotationUtil.sendMsgAsync(QuotationMsgEnum.REVIEW_PASS,
                Collections.singletonList(Long.valueOf(quotation.getCreateBy())),
                quotation.getQuotationNo());
        } else {
            // 审批通过 且非 销售确认
            if (auditParam.isPass()) {
                QuotationUtil.sendMsgAsync(QuotationMsgEnum.REVIEW, auditParam.getNextAuditUsers(),
                    quotation.getQuotationNo());
            } else {
                // 审批驳回
                QuotationUtil.sendMsgAsync(QuotationMsgEnum.REVIEW_REJECT,
                    Collections.singletonList(Long.valueOf(quotation.getCreateBy())),
                    quotation.getQuotationNo());
            }
        }

        switch (auditStatusEnum) {
            // 技术评审中
            case TECH_REVIEW_ELECTRIC:
            case TECH_REVIEW_MACHINE:
                // 技术评审通过
                for (Map.Entry<String, Object> entry : reviewResultJson.entrySet()) {
                    JSONArray from = JSONArray.from(entry.getValue());
                    for (int i = 0; i < from.size(); i++) {
                        JSONObject jsonObject = from.getJSONObject(i);
                        String solution = jsonObject.getString("solution");
                        String solutionMachine = jsonObject.getString("solutionMachine");
                        String quotationFlagElectric = jsonObject.getString("quotationFlagElectric");
                        String quotationFlagMachine = jsonObject.getString("quotationFlagMachine");
                        Long id = jsonObject.getLong("id");
                        if (StringUtils.isNotBlank(solution) || StringUtils.isNotBlank(solutionMachine)) {
                            if (Boolean.TRUE.equals(jsonObject.getBoolean("nonStandardDescFlag"))) {
                                tqSpecificationNonStandardService.update(
                                    Wrappers.<TqSpecificationNonStandard>lambdaUpdate()
                                        .eq(TqSpecificationNonStandard::getId, id)
                                        .set(TqSpecificationNonStandard::getSolution, solution)
                                        .set(TqSpecificationNonStandard::getSolutionMachine, solutionMachine)
                                        .set(TqSpecificationNonStandard::getQuotationFlagElectric,
                                            quotationFlagElectric)
                                        .set(TqSpecificationNonStandard::getQuotationFlagMachine,
                                            quotationFlagMachine));
                            } else {
                                priceDetailService.update(
                                    Wrappers.<TqPriceDetail>lambdaUpdate()
                                        .eq(TqPriceDetail::getId, id)
                                        .set(TqPriceDetail::getSolution, solution)
                                        .set(TqPriceDetail::getSolutionMachine, solutionMachine)
                                        .set(TqPriceDetail::getQuotationFlagElectric, quotationFlagElectric)
                                        .set(TqPriceDetail::getQuotationFlagMachine, quotationFlagMachine));
                            }
                        }
                    }
                }
                // 技术评审(机械) 且 通过
                if (auditParam.isPass() && auditStatusEnum.equals(AuditStatusEnum.TECH_REVIEW_MACHINE)) {
                    quotationStatus = QuotationStatusEnum.BUYER.getCode();
                }
                // 技术评审驳回
                if (!auditParam.isPass()) {
                    quotationStatus = QuotationStatusEnum.REVIEW_REJECT.getCode();
                }

                if (Objects.nonNull(quotationStatus)) {
                    // 更新报价
                    this.update(Wrappers.<TqQuotation>lambdaUpdate()
                        .eq(TqQuotation::getId, auditParam.getBusinessId())
                        .set(TqQuotation::getQuotationStatus, quotationStatus));
                    // 更新电梯规格
                    tqSpecificationService.update(Wrappers.<TqSpecification>lambdaUpdate()
                        .eq(TqSpecification::getQuotationId, auditParam.getBusinessId())
                        .eq(TqSpecification::getSystemCheckResult, SystemCheckResultEnum.OFF_STANDARD.getCode())
                        .set(TqSpecification::getQuotationStatus, quotationStatus)
                    );
                }
                break;
            // 采购报价中
            case BUYER_INQUIRY:
                for (Map.Entry<String, Object> entry : reviewResultJson.entrySet()) {
                    JSONArray from = JSONArray.from(entry.getValue());
                    for (int i = 0; i < from.size(); i++) {
                        JSONObject jsonObject = from.getJSONObject(i);
                        String buyerPrice = jsonObject.getString("buyerPrice");
                        String buyerPriceMachine = jsonObject.getString("buyerPriceMachine");
                        String quantity = jsonObject.getString("quantity");
                        Long id = jsonObject.getLong("id");
                        if (StringUtils.isBlank(quantity)) {
                            quantity = BigDecimal.ONE.toPlainString();
                        }
                        BigDecimal one = StringUtils.isNotBlank(buyerPrice)
                            ? new BigDecimal(buyerPrice) : BigDecimal.ZERO;
                        BigDecimal two = StringUtils.isNotBlank(buyerPriceMachine)
                            ? new BigDecimal(buyerPriceMachine) : BigDecimal.ZERO;
                        BigDecimal unitPriceTotal = one.add(two);
                        BigDecimal total = new BigDecimal(quantity)
                            .multiply(unitPriceTotal)
                            .setScale(0, RoundingMode.DOWN);
                        if (Boolean.TRUE.equals(jsonObject.getBoolean("nonStandardDescFlag"))) {
                            tqSpecificationNonStandardService.update(
                                Wrappers.<TqSpecificationNonStandard>lambdaUpdate()
                                    .eq(TqSpecificationNonStandard::getId, id)
                                    .set(TqSpecificationNonStandard::getBuyerPrice, buyerPrice)
                                    .set(TqSpecificationNonStandard::getBuyerPriceMachine, buyerPriceMachine)
                                    .set(TqSpecificationNonStandard::getTotal, total)
                            );
                        } else {
                            priceDetailService.update(Wrappers.<TqPriceDetail>lambdaUpdate()
                                .eq(TqPriceDetail::getId, id)
                                .set(TqPriceDetail::getBuyerPrice, buyerPrice)
                                .set(TqPriceDetail::getBuyerPriceMachine, buyerPriceMachine)
                                .set(TqPriceDetail::getTotal, total));
                        }
                    }
                }
                quotationStatus = QuotationStatusEnum.TREASURE.getCode();
                // 更新报价
                this.update(Wrappers.<TqQuotation>lambdaUpdate()
                    .eq(TqQuotation::getId, auditParam.getBusinessId())
                    .set(TqQuotation::getQuotationStatus, quotationStatus));
                // 更新电梯规格
                tqSpecificationService.update(Wrappers.<TqSpecification>lambdaUpdate()
                    .eq(TqSpecification::getQuotationId, auditParam.getBusinessId())
                    .eq(TqSpecification::getSystemCheckResult, SystemCheckResultEnum.OFF_STANDARD.getCode())
                    .set(TqSpecification::getQuotationStatus, quotationStatus)
                );
                break;
            // 财务报价中
            case TREASURE_INQUIRY:
                BigDecimal exchangeRate = quotation.getExchangeRate();
                // BigDecimal agentDiscountRate = quotation.getAgentDiscountRate();
                // BigDecimal countryDiscountRate = quotation.getCountryDiscountRate();
                // BigDecimal finalRate = getFinalRate(exchangeRate, agentDiscountRate, countryDiscountRate);

                // 电梯规格的非标选配价格总和MAP    KEY:specificationId  VALUE:非标选配价格总和
                Map<Long, BigDecimal> specificationPriceMap = new HashMap<>();
                // 报价的非标选配价格总和
                BigDecimal quotationOptionalPrice = BigDecimal.ZERO;
                for (Map.Entry<String, Object> entry : reviewResultJson.entrySet()) {
                    JSONArray from = JSONArray.from(entry.getValue());
                    for (int i = 0; i < from.size(); i++) {
                        JSONObject jsonObject = from.getJSONObject(i);
                        // String quantity = jsonObject.getString("quantity");
                        // String buyerPrice = jsonObject.getString("buyerPrice");
                        Long specificationId = jsonObject.getLong("specificationId");
                        String floatPercent = jsonObject.getString("floatPercent");
                        String floatPercentMachine = jsonObject.getString("floatPercentMachine");
                        String treasurePriceStr = jsonObject.getString("treasurerPrice");
                        String treasurePriceStrMachine = jsonObject.getString("treasurerPriceMachine");
                        String totalStr = jsonObject.getString("total");
                        String elevatorNumber = jsonObject.getString("elevatorNumber");
                        BigDecimal treasurePrice = StringUtils.isNotBlank(treasurePriceStr)
                            ? new BigDecimal(treasurePriceStr) : BigDecimal.ZERO;
                        BigDecimal treasurePriceMachine = StringUtils.isNotBlank(treasurePriceStrMachine)
                            ? new BigDecimal(treasurePriceStrMachine) : BigDecimal.ZERO;
                        BigDecimal treasurePriceTotal = treasurePrice.add(treasurePriceMachine);
                        BigDecimal total = new BigDecimal(totalStr);
                        Long id = jsonObject.getLong("id");

                        // BigDecimal treasurePrice = new BigDecimal(buyerPrice).multiply(
                        //     (new BigDecimal(floatPercent).add(BigDecimal.valueOf(100L)))
                        //         .divide(BigDecimal.valueOf(100L), 2, RoundingMode.HALF_UP)
                        // ).setScale(2, RoundingMode.DOWN);
                        //
                        // if (StringUtils.isBlank(quantity)) {
                        //     quantity = BigDecimal.ONE.toPlainString();
                        // }
                        //
                        // BigDecimal total = new BigDecimal(quantity)
                        //     .multiply(treasurePrice)
                        //     .setScale(2, RoundingMode.DOWN);

                        if (Boolean.TRUE.equals(jsonObject.getBoolean("nonStandardDescFlag"))) {
                            tqSpecificationNonStandardService.update(
                                Wrappers.<TqSpecificationNonStandard>lambdaUpdate()
                                    .eq(TqSpecificationNonStandard::getId, id)
                                    .set(TqSpecificationNonStandard::getFloatPercent, floatPercent)
                                    .set(TqSpecificationNonStandard::getFloatPercentMachine, floatPercentMachine)
                                    .set(TqSpecificationNonStandard::getTreasurerPrice, treasurePrice)
                                    .set(TqSpecificationNonStandard::getTreasurerPriceMachine, treasurePriceMachine)
                                    .set(TqSpecificationNonStandard::getPrice, treasurePriceTotal)
                                    .set(TqSpecificationNonStandard::getTotal, total));
                        } else {
                            priceDetailService.update(Wrappers.<TqPriceDetail>lambdaUpdate()
                                .eq(TqPriceDetail::getId, id)
                                .set(TqPriceDetail::getFloatPercent, floatPercent)
                                .set(TqPriceDetail::getFloatPercentMachine, floatPercentMachine)
                                .set(TqPriceDetail::getTreasurerPrice, treasurePrice)
                                .set(TqPriceDetail::getTreasurerPriceMachine, treasurePriceMachine)
                                .set(TqPriceDetail::getPrice, treasurePriceTotal)
                                .set(TqPriceDetail::getTotal, total));
                        }
                        total = total.multiply(exchangeRate).setScale(0, RoundingMode.DOWN);
                        quotationOptionalPrice = quotationOptionalPrice.add(
                            total.multiply(new BigDecimal(elevatorNumber)));
                        BigDecimal oneTotal = specificationPriceMap.getOrDefault(specificationId, BigDecimal.ZERO);
                        specificationPriceMap.put(specificationId, oneTotal.add(total));
                    }
                }
                // 更新电梯规格状态 及 选配价格
                for (Map.Entry<Long, BigDecimal> entry : specificationPriceMap.entrySet()) {
                    tqSpecificationMapper.updateAuditOptionalPrice(entry.getKey(), entry.getValue());
                }
                // 更新报价状态 及选配价格
                this.baseMapper.updateAuditOptionalPrice(auditParam.getBusinessId(), quotationOptionalPrice);
                break;
            // 销售确认
            case SALES_CONFIRM:
                quotationStatus = QuotationStatusEnum.ALREADY_BID.getCode();
                // 更新报价
                this.update(Wrappers.<TqQuotation>lambdaUpdate()
                    .eq(TqQuotation::getId, auditParam.getBusinessId())
                    .set(TqQuotation::getQuotationStatus, quotationStatus));
                // 更新电梯规格
                tqSpecificationService.update(Wrappers.<TqSpecification>lambdaUpdate()
                    .eq(TqSpecification::getQuotationId, auditParam.getBusinessId())
                    .eq(TqSpecification::getSystemCheckResult, SystemCheckResultEnum.OFF_STANDARD.getCode())
                    .set(TqSpecification::getQuotationStatus, quotationStatus));
                break;
            default:
                log.info("[报价] 未知状态 [{}]", auditParam.getCurrentStatus());
        }
    }

    @Override
    public BigDecimal getFinalRate(BigDecimal exchangeRate, BigDecimal agentDiscountRate,
        BigDecimal countryDiscountRate) {
        // 最终电梯折率  = 代理商折率 * 国家折率 * 币种汇率
        BigDecimal countryRate = countryDiscountRate
            .divide(new BigDecimal(ONE_HUNDRED), 10, RoundingMode.DOWN);
        BigDecimal finalRate = exchangeRate.multiply(countryRate);
        if (Objects.nonNull(agentDiscountRate)) {
            BigDecimal agentRate = agentDiscountRate
                .divide(new BigDecimal(ONE_HUNDRED), 10, RoundingMode.DOWN);
            finalRate = finalRate.multiply(agentRate);
        }
        return finalRate;
    }

    @Override
    public void changeBargainStatus(FlowAuditParam auditParam) {
        TqQuotation quotation = this.getById(auditParam.getBusinessId());
        if (AuditStatusEnum.INIT.getValue().equals(auditParam.getCurrentStatus())) {
            // 初始化审批流时,发邮件/站内信
            QuotationUtil.sendMsgAsync(QuotationMsgEnum.APPROVE, auditParam.getNextAuditUsers(),
                quotation.getQuotationNo());
            log.info("[{}] 未知节点状态", auditParam.getBusinessId());
            return;
        }
        Long bargainId = Optional.ofNullable(auditParam.getBusinessObj())
            .map(a -> a.getLong("bargainId")).orElse(0L);
        Long quotationId = Optional.ofNullable(auditParam.getBusinessObj())
            .map(a -> a.getLong("quotationId")).orElse(0L);
        String totalDiscountRate = Optional.ofNullable(auditParam.getBusinessObj())
            .map(a -> a.getString("discountRate")).orElse("");
        JSONArray tableList = Optional.ofNullable(auditParam.getBusinessObj())
            .map(a -> a.getJSONArray("tableList")).orElse(new JSONArray());

        // 报价当前状态
        AuditStatusEnum auditStatusEnum = AuditStatusEnum.from(auditParam.getCurrentStatus());
        NodeRoleEnum nodeRoleEnum = NodeRoleEnum.fromAuditStatusEnum(auditParam.getCurrentStatus());
        TbSalesDiscount salesDiscount;
        switch (auditStatusEnum) {
            // 议价审批(销售)
            case BARGAIN_APPROVAL1:
                // 议价审批(区总)
            case BARGAIN_APPROVAL2:
                // 议价审批(销售VP)
            case BARGAIN_APPROVAL3:
                salesDiscount = getRoleDiscount(nodeRoleEnum.getRoleId());
                log.info("议价审批获取角色折扣率  [{}] 区间 [{}]  [{}]",
                    nodeRoleEnum.getName(), salesDiscount.getMinDiscount(), salesDiscount.getMaxDiscount());
                for (int i = 0; i < tableList.size(); i++) {
                    JSONObject row = tableList.getJSONObject(i);
                    Long specificationId = row.getLong("id");
                    BigDecimal basicPrice = row.getBigDecimal("basicPrice");
                    BigDecimal originalBasicPrice = row.getBigDecimal("originalBasicPrice");
                    BigDecimal optionalPrice = row.getBigDecimal("optionalPrice");
                    BigDecimal discountRate = row.getBigDecimal("discountRate");
                    BigDecimal discountBasicPrice = originalBasicPrice.multiply(
                        discountRate.divide(new BigDecimal(100), 6, RoundingMode.HALF_UP));
                    BigDecimal discountPrice = discountBasicPrice.add(optionalPrice);
                    // 新增议价定价表
                    TqSalespersonSetPrice tqSalespersonSetPrice = new TqSalespersonSetPrice();
                    tqSalespersonSetPrice.setBargainId(bargainId);
                    tqSalespersonSetPrice.setSpecificationId(specificationId);
                    tqSalespersonSetPrice.setBasicPrice(basicPrice);
                    tqSalespersonSetPrice.setOptionalPrice(optionalPrice);
                    tqSalespersonSetPrice.setDiscountRate(discountRate);
                    tqSalespersonSetPrice.setDiscountbasicPrice(discountBasicPrice);
                    tqSalespersonSetPrice.setDiscountPrice(discountPrice);
                    tqSalespersonSetPrice.setDelFlag(DelFlagEnum.NORMAL.getValue());
                    tqSalespersonSetPrice.setCreateBy(SecurityUtils.getStrUserId());
                    tqSalespersonSetPrice.setCreateTime(DateUtils.getNowDate());
                    tqSalespersonSetPrice.setUpdateBy(SecurityUtils.getStrUserId());
                    tqSalespersonSetPrice.setUpdateTime(DateUtils.getNowDate());
                    tqSalespersonSetPriceService.save(tqSalespersonSetPrice);
                }
                String quotationStatus = null;
                // 节点通过
                if (auditParam.isPass()) {
                    BigDecimal number = new BigDecimal(totalDiscountRate);
                    if (number.compareTo(salesDiscount.getMinDiscount()) >= 0
                        && number.compareTo(salesDiscount.getMaxDiscount()) <= 0) {
                        // 结束审批
                        this.bargainPass(quotationId);
                        quotationStatus = QuotationStatusEnum.BARGAIN_CONFIRM.getCode();
                        // 消息通知    议价驳回（通知发起议价的人）
                        QuotationUtil.sendMsgAsync(QuotationMsgEnum.APPROVE_PASS,
                            Collections.singletonList(Long.valueOf(quotation.getCreateBy())),
                            quotation.getQuotationNo());
                    } else {
                        // 消息通知    议价通过 且  超出折率范围 （通知下个审批角色）
                        QuotationUtil.sendMsgAsync(QuotationMsgEnum.APPROVE,
                            auditParam.getNextAuditUsers(),
                            quotation.getQuotationNo());
                    }
                } else {
                    // 节点驳回
                    quotationStatus = QuotationStatusEnum.BARGAIN_REJECT.getCode();
                    // 消息通知    议价驳回（通知发起议价的人）
                    QuotationUtil.sendMsgAsync(QuotationMsgEnum.APPROVE_REJECT,
                        Collections.singletonList(Long.valueOf(quotation.getCreateBy())),
                        quotation.getQuotationNo());
                }
                if (Objects.nonNull(quotationStatus)) {
                    // 更新报价
                    this.update(Wrappers.<TqQuotation>lambdaUpdate()
                        .eq(TqQuotation::getId, quotationId)
                        .set(TqQuotation::getQuotationStatus, quotationStatus));
                    // 更新电梯规格
                    tqSpecificationService.update(Wrappers.<TqSpecification>lambdaUpdate()
                        .eq(TqSpecification::getQuotationId, quotationId)
                        .set(TqSpecification::getQuotationStatus, quotationStatus));
                }
                break;
            // 议价审批(总经理) 101
            case BARGAIN_APPROVAL4:
                // 审批通过 且 流程结束
                if (auditParam.isPass() && auditParam.isFinish()) {
                    this.bargainPass(quotationId);
                    // 消息通知    议价通过 （通知发起议价的人）
                    QuotationUtil.sendMsgAsync(QuotationMsgEnum.APPROVE_PASS,
                        Collections.singletonList(Long.valueOf(quotation.getCreateBy())),
                        quotation.getQuotationNo());
                }
                if (!auditParam.isPass()) {
                    // 驳回则追加vp审核及后续流程
                    flowService.insertCopy(auditParam,
                        Lists.newArrayList(BARGAIN_APPROVAL3.getValue(), BARGAIN_APPROVAL4.getValue()));
                    // 消息通知  议价驳回 且  总经理 （通知下个审批角色）
                    QuotationUtil.sendMsgAsync(QuotationMsgEnum.APPROVE, auditParam.getNextAuditUsers(),
                        quotation.getQuotationNo());
                }
                break;
        }
    }


    // 根据角色获取销售折扣
    @Override
    public TbSalesDiscount getRoleDiscount(Long roleId) {
        List<TbSalesDiscount> list = tbSalesDiscountService.list(Wrappers.<TbSalesDiscount>lambdaQuery()
            .eq(TbSalesDiscount::getRoleId, roleId)
            .eq(TbSalesDiscount::getDelFlag, DelFlagEnum.NORMAL.getValue())
        );
        if (CollUtil.isEmpty(list)) {
            throw new ServiceException("请先设置销售折率");
        } else {
            return list.get(0);
        }
    }

    private void bargainPass(Long quotationId) {
        String quotationStatus = QuotationStatusEnum.BARGAIN_CONFIRM.getCode();
        // 更新报价
        this.update(Wrappers.<TqQuotation>lambdaUpdate()
            .eq(TqQuotation::getId, quotationId)
            .set(TqQuotation::getQuotationStatus, quotationStatus));
        // 更新电梯规格
        tqSpecificationService.update(Wrappers.<TqSpecification>lambdaUpdate()
            .eq(TqSpecification::getQuotationId, quotationId)
            .set(TqSpecification::getQuotationStatus, quotationStatus));
        FlowAuditParam param = new FlowAuditParam();
        param.setAuditFlow((long)BARGAIN.getValue());
        param.setBusinessId(quotationId);
        flowService.finish(param);
    }

}
