package com.ruoyi.project.business.service.quotation.impl;

import static com.ruoyi.project.business.enums.AuditFlowEnum.BARGAIN;
import static com.ruoyi.project.business.enums.AuditFlowEnum.QUOTATION;
import static com.ruoyi.project.business.service.quotation.impl.QuoteApprovalServiceImpl.ONE_HUNDRED;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Pair;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import com.ruoyi.common.enums.FileBuzType;
import com.ruoyi.common.enums.SerialNumberPrefix;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.QuotationUtil;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.SpecialCharacterEscaper;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.framework.redis.RedisCache;
import com.ruoyi.project.business.domain.TbCurrency;
import com.ruoyi.project.business.domain.TbFile;
import com.ruoyi.project.business.domain.TbSalesDiscount;
import com.ruoyi.project.business.domain.TcProject;
import com.ruoyi.project.business.domain.TpElevatorModel;
import com.ruoyi.project.business.domain.TqBargainPrice;
import com.ruoyi.project.business.domain.TqPriceDetail;
import com.ruoyi.project.business.domain.TqQuotation;
import com.ruoyi.project.business.domain.TqSpecification;
import com.ruoyi.project.business.domain.TqSpecificationDetail;
import com.ruoyi.project.business.domain.TqSpecificationNonStandard;
import com.ruoyi.project.business.domain.bo.quotation.DescFileAddBO;
import com.ruoyi.project.business.domain.bo.quotation.ElevatorAddBO;
import com.ruoyi.project.business.domain.bo.quotation.ElevatorParamAddBO;
import com.ruoyi.project.business.domain.bo.quotation.NonStandardAddBO;
import com.ruoyi.project.business.domain.bo.quotation.QuotationAddBO;
import com.ruoyi.project.business.domain.bo.quotation.QuotationCopyBO;
import com.ruoyi.project.business.domain.bo.quotation.QuotationQueryBO;
import com.ruoyi.project.business.domain.flow.Flow;
import com.ruoyi.project.business.domain.vo.paramSelectItem.TpElevatorModelVo;
import com.ruoyi.project.business.domain.vo.quotation.CivilQuotationVO;
import com.ruoyi.project.business.domain.vo.quotation.H5ElevatorVO;
import com.ruoyi.project.business.domain.vo.quotation.H5QuotationDetailVO;
import com.ruoyi.project.business.domain.vo.quotation.QuotationApprovalVO;
import com.ruoyi.project.business.domain.vo.quotation.QuotationDetailVO;
import com.ruoyi.project.business.domain.vo.quotation.QuotationPriceVO;
import com.ruoyi.project.business.domain.vo.quotation.QuotationVO;
import com.ruoyi.project.business.domain.vo.quotation.QuotationVersionVO;
import com.ruoyi.project.business.domain.vo.selectParam.ProjectParam;
import com.ruoyi.project.business.dto.TaskGenerateParams;
import com.ruoyi.project.business.enums.DelFlagEnum;
import com.ruoyi.project.business.enums.NodeRoleEnum;
import com.ruoyi.project.business.enums.ParamTypeEnum;
import com.ruoyi.project.business.enums.QuotationEnum;
import com.ruoyi.project.business.enums.QuotationMsgEnum;
import com.ruoyi.project.business.enums.QuotationStatusEnum;
import com.ruoyi.project.business.enums.SystemCheckResultEnum;
import com.ruoyi.project.business.mapper.TbCurrencyMapper;
import com.ruoyi.project.business.mapper.TcProjectMapper;
import com.ruoyi.project.business.mapper.quotation.TqQuotationMapper;
import com.ruoyi.project.business.mapper.quotation.TqSpecificationMapper;
import com.ruoyi.project.business.service.ITbFileService;
import com.ruoyi.project.business.service.ITcAccountService;
import com.ruoyi.project.business.service.ITcLeadService;
import com.ruoyi.project.business.service.ITpElevatorModelService;
import com.ruoyi.project.business.service.flow.IFlowService;
import com.ruoyi.project.business.service.quotation.IPriceCalcService;
import com.ruoyi.project.business.service.quotation.IQuoteApprovalService;
import com.ruoyi.project.business.service.quotation.ITqBargainPriceService;
import com.ruoyi.project.business.service.quotation.ITqPriceDetailService;
import com.ruoyi.project.business.service.quotation.ITqQuotationService;
import com.ruoyi.project.business.service.quotation.ITqSpecificationDetailService;
import com.ruoyi.project.business.service.quotation.ITqSpecificationNonStandardService;
import com.ruoyi.project.business.service.quotation.ITqSpecificationService;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author ruoyi
 * @date 2024-05-10
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class TqQuotationServiceImpl extends ServiceImpl<TqQuotationMapper, TqQuotation> implements ITqQuotationService {

    public static final DecimalFormat formatter = new DecimalFormat("#,###");

    private final TqSpecificationMapper tqSpecificationMapper;

    private final ITqSpecificationService tqSpecificationService;

    private final TcProjectMapper tcProjectMapper;

    private final RedisCache redisCache;

    private final ITqSpecificationDetailService tqSpecificationDetailService;

    private final IFlowService flowService;

    private final IQuoteApprovalService quoteApprovalService;

    private final ITqSpecificationNonStandardService tqSpecificationNonStandardService;

    private final ITqPriceDetailService priceDetailService;

    private final ITqBargainPriceService tqBargainPriceService;

    private final ITbFileService tbFileService;

    private final ITcLeadService tcLeadService;

    private final ITpElevatorModelService tpElevatorModelService;

    private final IPriceCalcService priceCalcService;

    private final ITcAccountService tcAccountService;

    private final TbCurrencyMapper tbCurrencyMapper;

    /**
     * 查询【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public QuotationDetailVO selectTqQuotationById(Long id) {
        QuotationDetailVO quotationDetailVO = baseMapper.selectDetailById(id);
        ProjectParam tcProject = new ProjectParam();
        tcProject.setProjectId(quotationDetailVO.getProjectId());
        tcProject.setPermissionChar(SecurityUtils.getPermissionCharSea());
        List<TcProject> tcProjects = tcProjectMapper.selectTcProjectList(tcProject);
        if (CollUtil.isNotEmpty(tcProjects)) {
            BeanUtils.copyProperties(tcProjects.get(0), quotationDetailVO, "currencyId", "updateTime");
        }
        return quotationDetailVO;
    }

    /**
     * 查询【请填写功能名称】列表
     *
     * @param queryBO 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<QuotationVO> selectTqQuotationList(QuotationQueryBO queryBO) {
        SpecialCharacterEscaper.escapeSpecialCharacters(queryBO
            , QuotationQueryBO::getQuotationNo
            , QuotationQueryBO::getProjectName
            , QuotationQueryBO::getCity
            , QuotationQueryBO::getElevatorModel
            , QuotationQueryBO::getNickName
        );
        String permissionCharIns = SecurityUtils.getPermissionCharIns();
        // queryBO.setAreaIdList(tbAreaService.selectChildAreaIdsByAreaIds(SecurityUtils.getLoginUser().getAreaIdList()));
        queryBO.setPermissionChar(permissionCharIns);
        // 代理商销售经理的场合
        if (SecurityUtils.hasRole("agent_sales")) {
            queryBO.setLoginUserId(SecurityUtils.getUserId());
            queryBO.setPermission(SecurityUtils.getPermissionCharIns());
        }
        List<QuotationVO> quotationVOS = baseMapper.selectTqQuotationList(queryBO);

        // 查询子项
        List<String> quotationNoList = quotationVOS.stream().map(QuotationVO::getQuotationNo)
            .collect(Collectors.toList());
        List<Long> quotatiionIdList = quotationVOS.stream().map(QuotationVO::getId)
            .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(quotationNoList) && queryBO.getIsGroupBy()) {
            QuotationQueryBO quotationQueryBO = new QuotationQueryBO();
            quotationQueryBO.setQuotationNoList(quotationNoList);
            quotationQueryBO.setExcludeQuotationIdList(quotatiionIdList);
            quotationQueryBO.setIsGroupBy(false);
            quotationQueryBO.setPermissionChar(permissionCharIns);
            // 代理商销售经理的场合
            if (SecurityUtils.hasRole("agent_sales")) {
                quotationQueryBO.setLoginUserId(SecurityUtils.getUserId());
                quotationQueryBO.setPermission(SecurityUtils.getPermissionCharIns());
            }
            List<QuotationVO> childrenList = baseMapper.selectTqQuotationList(quotationQueryBO);
            if (CollUtil.isNotEmpty(childrenList)) {
                // 获取审批节点信息
                List<Long> collect = childrenList.stream().map(QuotationVO::getId)
                    .collect(Collectors.toList());
                Map<Long, Flow> flowMap = flowService.getFlowMap(collect, Arrays.asList(QUOTATION, BARGAIN));
                childrenList.forEach(item -> {
                    if (Objects.nonNull(item.getAdditionalFees())) {
                        item.setAdditionalFees(
                            item.getAdditionalFees().multiply(item.getExchangeRate().setScale(0, RoundingMode.DOWN)));
                    }
                    item.setFlow(flowMap.get(item.getId()));
                });

                if (CollUtil.isNotEmpty(childrenList)) {
                    Map<String, List<QuotationVO>> childrenMap = childrenList.stream()
                        .collect(Collectors.groupingBy(QuotationVO::getQuotationNo));
                    for (QuotationVO quotationVO : quotationVOS) {
                        quotationVO.setSelectable(true);
                        quotationVO.setChildren(childrenMap.getOrDefault(quotationVO.getQuotationNo(),
                            Collections.emptyList()));
                    }
                }
            }
        }

        // 获取审批节点信息
        Map<Long, Flow> flowMap = flowService.getFlowMap(quotatiionIdList, Arrays.asList(QUOTATION, BARGAIN));
        quotationVOS.forEach(item -> {
            if (Objects.nonNull(item.getAdditionalFees())) {
                item.setAdditionalFees(
                    item.getAdditionalFees().multiply(item.getExchangeRate().setScale(0, RoundingMode.DOWN)));
            }
            item.setFlow(flowMap.get(item.getId()));
        });

        return quotationVOS;
    }

    /**
     * 新增【请填写功能名称】
     *
     * @param quotationAddBO 【请填写功能名称】
     * @return 结果
     */
    @Override
    @Transactional
    public void insertTqQuotation(QuotationAddBO quotationAddBO) {
        // 获取币种汇率
        List<TbCurrency> tbCurrencies = tbCurrencyMapper.selectList(Wrappers.<TbCurrency>lambdaQuery()
            .eq(TbCurrency::getCurrencyName, quotationAddBO.getCurrencyId())
            .le(TbCurrency::getEffectiveDate, DateUtils.getNowDate())
            .ge(TbCurrency::getExpirationDate, DateUtils.getNowDate())
            .eq(TbCurrency::getDelFlag, DelFlagEnum.NORMAL.getValue())
            .orderByDesc(TbCurrency::getCurrencyId));
        if (CollUtil.isNotEmpty(tbCurrencies)) {
            TbCurrency currency = tbCurrencies.get(0);
            quotationAddBO.setExchangeRate(currency.getExchangeRate());
        } else {
            throw new ServiceException(MessageUtils.message("quotation.currency.unknown"));
        }

        // 验证电梯是否属于关联项目
        List<Pair<Long, String>> collect = quotationAddBO.getElevatorList().stream()
            .map(item -> Pair.of(item.getElevatorModelId(), item.getElevatorModel()))
            .collect(Collectors.toList());
        TcProject tcProjects = tcProjectMapper.selectTcProjectByProjectId(quotationAddBO.getProjectId());
        if (Objects.nonNull(tcProjects)) {
            checkAreaElevatorCorrelation(tcProjects.getCountryId(), collect);
        } else {
            log.info("项目不存在 [{}]", quotationAddBO.getProjectId());
        }
        // 报价
        TqQuotation tqQuotation = new TqQuotation();
        String quotationNo = redisCache.generateSerialNumber(SerialNumberPrefix.BJ.getPrefix());
        tqQuotation.setQuotationNo(quotationNo);
        tqQuotation.setVersion(1L);
        tqQuotation.setAgentId(quotationAddBO.getAgentId());
        tqQuotation.setExchangeRate(quotationAddBO.getExchangeRate());
        tqQuotation.setAgentDiscountRate(quotationAddBO.getAgentDiscountRate());
        tqQuotation.setCountryDiscountRate(quotationAddBO.getCountryDiscountRate());
        tqQuotation.setProjectId(quotationAddBO.getProjectId());
        tqQuotation.setCurrencyId(quotationAddBO.getCurrencyId());
        tqQuotation.setSubmitTime(new Date());
        tqQuotation.setQuotationStatus(quotationAddBO.getQuotationStatus());
        tqQuotation.setDelFlag(DelFlagEnum.NORMAL.getValue());
        tqQuotation.setCreateBy(SecurityUtils.getStrUserId());
        tqQuotation.setCreateTime(DateUtils.getNowDate());
        tqQuotation.setUpdateBy(SecurityUtils.getStrUserId());
        tqQuotation.setUpdateTime(DateUtils.getNowDate());
        // 区域ID
        List<Long> areaIdList = baseMapper.getAreaIdByProjectId(quotationAddBO.getProjectId());
        if (CollUtil.isNotEmpty(areaIdList)) {
            tqQuotation.setAreaId(areaIdList.get(0));
        } else {
            throw new ServiceException(MessageUtils.message("quotation.unknown.area"));
        }
        String permissionChar = tcAccountService.getPermissionChar(tqQuotation.getAreaId());
        // 数据权限字符
        tqQuotation.setPermissionChar(permissionChar);
        baseMapper.insertTqQuotation(tqQuotation);
        Long quotationId = tqQuotation.getId();
        log.info("报价id [{}]", quotationId);
        boolean quotationStandardFlag = insertSpecificationParam(quotationAddBO
            , quotationId
            , isNeedCalc(quotationAddBO.getQuotationStatus()));
        // 技术评审中时，启动审批流程（技术评审）
        if (QuotationStatusEnum.REVIEWING.getCode().equals(quotationAddBO.getQuotationStatus())) {
            if (quotationStandardFlag) {
                this.update(Wrappers.<TqQuotation>lambdaUpdate()
                    .eq(TqQuotation::getId, quotationId)
                    .set(TqQuotation::getQuotationStatus, QuotationStatusEnum.ALREADY_BID.getCode())
                );
                // 发送邮件通知
                QuotationUtil.sendMsgAsync(QuotationMsgEnum.STAND_QUOTATION,
                    Collections.singletonList(SecurityUtils.getUserId()),
                    quotationNo);
            } else {
                flowService.generateTasks(
                    new TaskGenerateParams(QUOTATION, tqQuotation, SecurityUtils.getUserId(),
                        tqQuotation.getQuotationNo()));
            }
            tcLeadService.convertQuotationFromProject(quotationAddBO.getProjectId(), quotationId);
        }
    }

    @Override
    public void checkAreaElevatorCorrelation(Long countryId, List<Pair<Long, String>> elevatorList) {
        TpElevatorModelVo tpElevatorModel = new TpElevatorModelVo();
        tpElevatorModel.setCountryId(countryId);
        tpElevatorModel.setUserId(SecurityUtils.getUserId());
        List<TpElevatorModel> tpElevatorModels = tpElevatorModelService.listNewAll(tpElevatorModel);
        List<Long> allocated = tpElevatorModels.stream().map(TpElevatorModel::getId).collect(Collectors.toList());
        Set<String> set = new HashSet<>();
        elevatorList.forEach(item -> {
            if (!allocated.contains(item.getKey())) {
                set.add(item.getValue());
            }
        });
        if (set.size() > 0) {
            throw new ServiceException(MessageUtils.message("area.elevator.mismatch",
                StringUtils.join(set, ",")));
        }
    }

    // 新增电梯规则 和 参数
    private boolean insertSpecificationParam(QuotationAddBO quotationAddBO, Long quotationId, boolean needCalc) {
        BigDecimal finalRate = quoteApprovalService.getFinalRate(quotationAddBO.getExchangeRate(),
            quotationAddBO.getAgentDiscountRate(),
            quotationAddBO.getCountryDiscountRate());
        // 基础价格 （含国家折扣 代理商折扣 汇率）
        BigDecimal quotationBasicPrice = BigDecimal.ZERO;
        // 基础价格 （含汇率）
        BigDecimal quotationOriginalBasicPrice = BigDecimal.ZERO;
        // 选配价格 （含汇率）
        BigDecimal quotationOptionalPrice = BigDecimal.ZERO;
        boolean quotationStandardFlag = true;
        for (ElevatorAddBO elevator : quotationAddBO.getElevatorList()) {
            boolean elevatorStandardFlag = true;
            TqSpecification tqSpecification = new TqSpecification();
            tqSpecification.setQuotationId(quotationId);
            tqSpecification.setElevatorModelId(elevator.getElevatorModelId());
            tqSpecification.setElevatorNumber(elevator.getElevatorNumber());
            tqSpecification.setElevatorGroupCode(elevator.getElevatorGroupCode());
            tqSpecification.setElevatorCode(elevator.getElevatorCode());
            tqSpecification.setSpecification(elevator.getSpecification());
            tqSpecification.setQuotationStatus(quotationAddBO.getQuotationStatus());
            tqSpecification.setSystemCheckResult(SystemCheckResultEnum.OFF_STANDARD.getCode());
            tqSpecification.setDelFlag(DelFlagEnum.NORMAL.getValue());
            tqSpecification.setCreateBy(SecurityUtils.getStrUserId());
            if (Objects.nonNull(elevator.getCreateTime1())) {
                tqSpecification.setCreateTime(elevator.getCreateTime1());
            } else {
                tqSpecification.setCreateTime(DateUtils.getNowDate());
            }
            tqSpecification.setUpdateBy(SecurityUtils.getStrUserId());
            tqSpecification.setUpdateTime(DateUtils.getNowDate());
            Long basicPriceLong = 0L;
            List<TqPriceDetail> tqPriceDetailList = new ArrayList<>();
            if (needCalc) {
                // 初始化梯形计算参数
                QuotationUtil.initMap(elevator.getParamList());
                try {
                    // 计算电梯基础价格 &&  层站差价放到选配价格
                    basicPriceLong = priceCalcService.calculateBasicPrice(elevator, tqPriceDetailList);
                    log.info("基础价格 [{}]", basicPriceLong);
                } catch (Exception e) {
                    log.error("计算基价失败", e);
                    throw new ServiceException(
                        MessageUtils.message("elevator.basic.price.fail",
                            elevator.getElevatorModel(),
                            elevator.getSpecification()));
                }
                // 计算电梯选配价格 （以及价格明细）
                try {
                    priceCalcService.statisticsOptionalPrice(elevator, tqPriceDetailList);
                } catch (Exception e) {
                    log.error("计算加价项目失败", e);
                    throw new ServiceException(
                        MessageUtils.message("elevator.optional.price.fail",
                            elevator.getElevatorModel(),
                            elevator.getSpecification()));
                }
            }
            // 选配价格求和
            BigDecimal optionalPrice = tqPriceDetailList.stream()
                .filter(item -> !item.getMarkupName().equals(PriceCalcServiceImpl.BASIC_PRICE))
                .map(TqPriceDetail::getTotal)
                .filter(Objects::nonNull)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
            log.info("选配价格 [{}]", optionalPrice);
            BigDecimal decimalBasicPrice = BigDecimal.valueOf(basicPriceLong);
            // 电梯单价乘最终折率
            BigDecimal basicPrice = decimalBasicPrice.multiply(finalRate).setScale(0, RoundingMode.DOWN);
            BigDecimal originalBasicPrice = decimalBasicPrice.multiply(quotationAddBO.getExchangeRate())
                .setScale(0, RoundingMode.DOWN);
            optionalPrice = optionalPrice.multiply(quotationAddBO.getExchangeRate()).setScale(0, RoundingMode.DOWN);
            tqSpecification.setBasicPrice(basicPrice);
            tqSpecification.setOriginalBasicPrice(originalBasicPrice);
            tqSpecification.setOptionalPrice(optionalPrice);
            tqSpecification.setPrice(basicPrice.add(optionalPrice));
            // 判断规则加价项目是否有非标
            if (tqPriceDetailList.stream().anyMatch(a ->
                a.getSystemCheckResult().equals(SystemCheckResultEnum.OFF_STANDARD.getCode()))
                || CollUtil.isNotEmpty(elevator.getNonStandardList())) {
                elevatorStandardFlag = false;
            } else {
                tqSpecification.setQuotationStatus(QuotationStatusEnum.ALREADY_BID.getCode());
                tqSpecification.setSystemCheckResult(SystemCheckResultEnum.STANDARD.getCode());
            }
            // 如果是草稿,规格状态改为草稿
            if (QuotationStatusEnum.DRAFT.getCode().equals(quotationAddBO.getQuotationStatus())) {
                tqSpecification.setQuotationStatus(QuotationStatusEnum.DRAFT.getCode());
            }
            tqSpecificationService.insertTqSpecification(tqSpecification);
            // 统计报价的基础价格 选配价格
            quotationOriginalBasicPrice = quotationOriginalBasicPrice.add(
                tqSpecification.getOriginalBasicPrice()
                    .multiply(BigDecimal.valueOf(tqSpecification.getElevatorNumber())));
            quotationBasicPrice = quotationBasicPrice.add(
                tqSpecification.getBasicPrice().multiply(BigDecimal.valueOf(tqSpecification.getElevatorNumber())));
            quotationOptionalPrice = quotationOptionalPrice.add(
                tqSpecification.getOptionalPrice().multiply(BigDecimal.valueOf(tqSpecification.getElevatorNumber())));

            Long specificationId = tqSpecification.getId();
            log.info("规格id [{}]", specificationId);
            // 保存价格明细
            if (CollUtil.isNotEmpty(tqPriceDetailList)) {
                tqPriceDetailList = tqPriceDetailList.stream().peek(item -> {
                    item.setSpecificationId(specificationId);
                }).collect(Collectors.toList());
                priceDetailService.saveBatch(tqPriceDetailList);
            }

            // 非标记录表
            List<NonStandardAddBO> nonStandardList = elevator.getNonStandardList();
            if (CollUtil.isNotEmpty(nonStandardList)) {
                elevatorStandardFlag = false;
                for (NonStandardAddBO nonStandardAddBO : nonStandardList) {
                    TqSpecificationNonStandard tqSpecificationNonStandard = new TqSpecificationNonStandard();
                    tqSpecificationNonStandard.setSpecificationId(specificationId);
                    tqSpecificationNonStandard.setNonStandardDesc(nonStandardAddBO.getNonStandardDesc());
                    tqSpecificationNonStandard.setSystemCheckResult(SystemCheckResultEnum.OFF_STANDARD.getCode());
                    tqSpecificationNonStandard.setDelFlag(DelFlagEnum.NORMAL.getValue());
                    tqSpecificationNonStandard.setQuotationFlagElectric(QuotationEnum.NO.getValue());
                    tqSpecificationNonStandard.setQuotationFlagMachine(QuotationEnum.NO.getValue());
                    tqSpecificationNonStandard.setCreateBy(SecurityUtils.getStrUserId());
                    tqSpecificationNonStandard.setCreateTime(DateUtils.getNowDate());
                    tqSpecificationNonStandard.setUpdateBy(SecurityUtils.getStrUserId());
                    tqSpecificationNonStandard.setUpdateTime(DateUtils.getNowDate());
                    tqSpecificationNonStandardService.insertTqSpecificationNonStandard(tqSpecificationNonStandard);
                    // 上传文件
                    List<DescFileAddBO> fileList = nonStandardAddBO.getFileList();
                    if (CollUtil.isNotEmpty(fileList)) {
                        List<TbFile> tbFileList = new ArrayList<>();
                        for (DescFileAddBO descFileAddBO : fileList) {
                            TbFile tbFile = new TbFile();
                            tbFile.setBusinessId(tqSpecificationNonStandard.getId());
                            tbFile.setBusinessType(FileBuzType.NON_STANDARD_DESC.getType());
                            tbFile.setRealFileName(descFileAddBO.getRealFileName());
                            tbFile.setServerFileName(descFileAddBO.getServerFileName());
                            tbFile.setFilePath(descFileAddBO.getFilePath());
                            tbFile.setDelFlag(DelFlagEnum.NORMAL.getValue());
                            tbFile.setCreateTime(DateUtils.getNowDate());
                            tbFile.setCreateBy(SecurityUtils.getStrUserId());
                            tbFile.setUpdateTime(DateUtils.getNowDate());
                            tbFile.setUpdateBy(SecurityUtils.getStrUserId());
                            tbFileList.add(tbFile);
                        }
                        tbFileService.saveBatch(tbFileList);
                    }
                }
            }

            // 过滤出来普通参数
            List<ElevatorParamAddBO> normalParamList = elevator.getParamList().stream()
                .filter(item -> ParamTypeEnum.NORMAL.getCode().equals(item.getParamType())).collect(
                    Collectors.toList());
            for (ElevatorParamAddBO paramAddBO : normalParamList) {
                TqSpecificationDetail tqSpecificationDetail = new TqSpecificationDetail();
                tqSpecificationDetail.setSpecificationId(specificationId);
                tqSpecificationDetail.setParaCode(paramAddBO.getParaCode());
                tqSpecificationDetail.setParaValue(paramAddBO.getParaValue());
                tqSpecificationDetail.setParaCategory(paramAddBO.getParaCategory());
                tqSpecificationDetail.setDelFlag(DelFlagEnum.NORMAL.getValue());
                tqSpecificationDetail.setCreateBy(SecurityUtils.getStrUserId());
                tqSpecificationDetail.setCreateTime(DateUtils.getNowDate());
                tqSpecificationDetail.setUpdateBy(SecurityUtils.getStrUserId());
                tqSpecificationDetail.setUpdateTime(DateUtils.getNowDate());
                tqSpecificationDetailService.insertTqSpecificationDetail(tqSpecificationDetail);
            }
            // 过滤参数包参数
            List<ElevatorParamAddBO> parabarParamList = elevator.getParamList().stream()
                .filter(item -> ParamTypeEnum.BAR_PARAM.getCode().equals(item.getParamType())).collect(
                    Collectors.toList());
            for (ElevatorParamAddBO paramAddBO : parabarParamList) {
                TqSpecificationDetail tqSpecificationDetail = new TqSpecificationDetail();
                tqSpecificationDetail.setSpecificationId(specificationId);
                tqSpecificationDetail.setParaCode(paramAddBO.getParaCode());
                tqSpecificationDetail.setParaValue(paramAddBO.getParaValue());
                tqSpecificationDetail.setParabarId(paramAddBO.getParabarId());
                tqSpecificationDetail.setParabarDetailId(paramAddBO.getParabarDetailId());
                tqSpecificationDetail.setParaCategory(paramAddBO.getParaCategory());
                tqSpecificationDetail.setDelFlag(DelFlagEnum.NORMAL.getValue());
                tqSpecificationDetail.setCreateBy(SecurityUtils.getStrUserId());
                tqSpecificationDetail.setCreateTime(DateUtils.getNowDate());
                tqSpecificationDetail.setUpdateBy(SecurityUtils.getStrUserId());
                tqSpecificationDetail.setUpdateTime(DateUtils.getNowDate());
                tqSpecificationDetailService.insertTqSpecificationDetail(tqSpecificationDetail);
            }
            // 规格有一个非标，报价就是非标
            if (!elevatorStandardFlag) {
                quotationStandardFlag = false;
            }
        }
        // 根据规格更新价格
        BigDecimal quotationPrice = quotationBasicPrice.add(quotationOptionalPrice);
        this.update(Wrappers.<TqQuotation>lambdaUpdate()
            .eq(TqQuotation::getId, quotationId)
            .set(TqQuotation::getBasicPrice, quotationBasicPrice)
            .set(TqQuotation::getOriginalBasicPrice, quotationOriginalBasicPrice)
            .set(TqQuotation::getOptionalPrice, quotationOptionalPrice)
            .set(TqQuotation::getPrice, quotationPrice)
        );
        return quotationStandardFlag;
    }

    /**
     * 修改【请填写功能名称】
     *
     * @param quotationAddBO 【请填写功能名称】
     * @return 结果
     */
    @Override
    @Transactional
    public void updateTqQuotation(QuotationAddBO quotationAddBO) {
        Long quotationId = quotationAddBO.getQuotationId();
        // 报价更新状态
        TqQuotation tqQuotation = this.getById(quotationId);
        this.update(Wrappers.<TqQuotation>lambdaUpdate()
            .eq(TqQuotation::getId, quotationId)
            .eq(TqQuotation::getQuotationStatus, QuotationStatusEnum.DRAFT.getCode())
            .set(TqQuotation::getQuotationStatus, quotationAddBO.getQuotationStatus())
            .set(TqQuotation::getUpdateBy, SecurityUtils.getStrUserId())
            .set(TqQuotation::getUpdateTime, DateUtils.getNowDate()));

        // 电梯规格删除
        List<TqSpecification> tqSpecifications = tqSpecificationService.list(
            Wrappers.<TqSpecification>lambdaQuery()
                .eq(TqSpecification::getQuotationId, quotationId)
                .eq(TqSpecification::getDelFlag, DelFlagEnum.NORMAL.getValue()));
        List<Long> specificationIdList = tqSpecifications.stream().map(TqSpecification::getId)
            .collect(Collectors.toList());
        tqSpecificationMapper.deleteByQuotationId(quotationAddBO.getQuotationId());
        // 删除梯形参数,非标描述
        if (CollUtil.isNotEmpty(specificationIdList)) {
            // 梯形参数
            tqSpecificationDetailService.update(Wrappers.<TqSpecificationDetail>lambdaUpdate()
                .in(TqSpecificationDetail::getSpecificationId, specificationIdList)
                .set(TqSpecificationDetail::getDelFlag, DelFlagEnum.DELETED.getValue()));
            // 非标描述
            tqSpecificationNonStandardService.update(
                Wrappers.<TqSpecificationNonStandard>lambdaUpdate()
                    .in(TqSpecificationNonStandard::getSpecificationId, specificationIdList)
                    .set(TqSpecificationNonStandard::getDelFlag, DelFlagEnum.DELETED.getValue()));
            // 价格明细
            priceDetailService.update(
                Wrappers.<TqPriceDetail>lambdaUpdate()
                    .in(TqPriceDetail::getSpecificationId, specificationIdList)
                    .set(TqPriceDetail::getDelFlag, DelFlagEnum.DELETED.getValue())
            );
        }
        quotationAddBO.setExchangeRate(tqQuotation.getExchangeRate());
        quotationAddBO.setCountryDiscountRate(tqQuotation.getCountryDiscountRate());
        quotationAddBO.setAgentDiscountRate(tqQuotation.getAgentDiscountRate());
        boolean quotationStandardFlag = insertSpecificationParam(quotationAddBO
            , quotationId
            , isNeedCalc(quotationAddBO.getQuotationStatus()));
        // 技术评审中时，启动审批流程（技术评审）
        if (QuotationStatusEnum.REVIEWING.getCode().equals(quotationAddBO.getQuotationStatus())) {
            if (quotationStandardFlag) {
                this.update(Wrappers.<TqQuotation>lambdaUpdate()
                    .eq(TqQuotation::getId, quotationId)
                    .set(TqQuotation::getQuotationStatus, QuotationStatusEnum.ALREADY_BID.getCode())
                );
                // 发送邮件通知
                QuotationUtil.sendMsgAsync(QuotationMsgEnum.STAND_QUOTATION,
                    Collections.singletonList(SecurityUtils.getUserId()),
                    tqQuotation.getQuotationNo());
            } else {
                flowService.generateTasks(
                    new TaskGenerateParams(QUOTATION, tqQuotation, SecurityUtils.getUserId(),
                        tqQuotation.getQuotationNo()));
            }
            tcLeadService.convertQuotationFromProject(quotationAddBO.getProjectId(), quotationId);
        }
    }

    // 是否计算梯形价格(上线后放开注释)
    private boolean isNeedCalc(String quotationStatus) {
        return true;
        // return !QuotationStatusEnum.DRAFT.getCode().equals(quotationStatus);
    }

    /**
     * 批量删除【请填写功能名称】
     *
     * @param ids 需要删除的【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteTqQuotationByIds(Long[] ids) {
        return baseMapper.deleteTqQuotationByIds(ids);
    }

    /**
     * 删除【请填写功能名称】信息
     *
     * @param id 【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteTqQuotationById(Long id) {
        return baseMapper.deleteTqQuotationById(id);
    }

    @Override
    @Transactional
    public void copyVersions(QuotationCopyBO copyBO) {
        List<TqQuotation> tqQuotations = baseMapper.selectBatchIds(copyBO.getIds());
        // 获取报价编号的最大版本号
        List<QuotationVersionVO> maxVersionById = baseMapper.getMaxVersionById(copyBO.getIds());
        Map<String, Long> quotationVersionMap = maxVersionById.stream()
            .collect(Collectors.toMap(QuotationVersionVO::getQuotationNo, QuotationVersionVO::getVersion, (a, b) -> a));
        // 报价复制版本
        for (TqQuotation tqQuotation : tqQuotations) {
            // 复制前报价ID
            Long oldQuotationId = tqQuotation.getId();
            // 复制报价
            Long nextVersion = quotationVersionMap.getOrDefault(tqQuotation.getQuotationNo(), 1L) + 1;
            quotationVersionMap.put(tqQuotation.getQuotationNo(), nextVersion);
            tqQuotation.setVersion(nextVersion);
            tqQuotation.setQuotationStatus(QuotationStatusEnum.DRAFT.getCode());
            tqQuotation.setPrice(null);
            String permissionChar = tcAccountService.getPermissionChar(tqQuotation.getAreaId());
            tqQuotation.setPermissionChar(permissionChar);
            tqQuotation.setFreightPrice(null);
            tqQuotation.setBasicPrice(null);
            tqQuotation.setOptionalPrice(null);
            tqQuotation.setFinalPrice(null);
            tqQuotation.setFinalCurrencyPrice(null);
            tqQuotation.setSubmitTime(DateUtils.getNowDate());
            tqQuotation.setCreateTime(DateUtils.getNowDate());
            tqQuotation.setCreateBy(SecurityUtils.getStrUserId());
            tqQuotation.setUpdateTime(DateUtils.getNowDate());
            tqQuotation.setUpdateBy(SecurityUtils.getStrUserId());
            baseMapper.insertTqQuotation(tqQuotation);
            // 复制后报价ID
            Long newQuotationId = tqQuotation.getId();
            // 复制电梯规格
            List<TqSpecification> tqSpecifications = tqSpecificationService.list(
                Wrappers.<TqSpecification>lambdaQuery()
                    .eq(TqSpecification::getQuotationId, oldQuotationId)
                    .eq(TqSpecification::getDelFlag, DelFlagEnum.NORMAL.getValue())
            );
            tqSpecifications.forEach(item -> {
                item.setQuotationId(newQuotationId);
                item.setPrice(null);
                item.setBasicPrice(null);
                item.setOptionalPrice(null);
                item.setFinalPrice(null);
                item.setQuotationStatus(QuotationStatusEnum.DRAFT.getCode());
                item.setCreateTime(DateUtils.getNowDate());
                item.setCreateBy(SecurityUtils.getStrUserId());
                item.setUpdateTime(DateUtils.getNowDate());
                item.setUpdateBy(SecurityUtils.getStrUserId());
            });
            // 电梯规格复制版本
            for (TqSpecification tqSpecification : tqSpecifications) {
                // 复制前规格ID
                Long oldSpecificationId = tqSpecification.getId();
                tqSpecificationService.insertTqSpecification(tqSpecification);
                // 复制后规格ID
                Long newSpecificationId = tqSpecification.getId();
                // 复制电梯规格参数
                List<TqSpecificationDetail> paramDetail = tqSpecificationDetailService.list(
                    Wrappers.<TqSpecificationDetail>lambdaQuery()
                        .eq(TqSpecificationDetail::getSpecificationId, oldSpecificationId)
                        .eq(TqSpecificationDetail::getDelFlag, DelFlagEnum.NORMAL.getValue())
                );
                // 电梯规格参数复制版本
                paramDetail.forEach(item -> {
                    item.setSpecificationId(newSpecificationId);
                    item.setId(null);
                });
                tqSpecificationDetailService.saveBatch(paramDetail);
                // 复制非标描述
                List<TqSpecificationNonStandard> nonStandardList = tqSpecificationNonStandardService.list(
                    Wrappers.<TqSpecificationNonStandard>lambdaQuery()
                        .eq(TqSpecificationNonStandard::getSpecificationId, oldSpecificationId)
                        .eq(TqSpecificationNonStandard::getDelFlag, DelFlagEnum.NORMAL.getValue())
                );
                for (TqSpecificationNonStandard tqSpecificationNonStandard : nonStandardList) {
                    // 复制前非标描述ID
                    Long oldNonStandardId = tqSpecificationNonStandard.getId();
                    tqSpecificationNonStandard.setSpecificationId(newSpecificationId);
                    tqSpecificationNonStandard.setBuyerPrice(null);
                    tqSpecificationNonStandard.setFloatPercent(null);
                    tqSpecificationNonStandard.setTreasurerPrice(null);
                    tqSpecificationNonStandard.setPrice(null);
                    tqSpecificationNonStandard.setTotal(null);
                    tqSpecificationNonStandard.setSpecificationId(newSpecificationId);
                    tqSpecificationNonStandard.setCreateTime(DateUtils.getNowDate());
                    tqSpecificationNonStandard.setCreateBy(SecurityUtils.getStrUserId());
                    tqSpecificationNonStandard.setUpdateTime(DateUtils.getNowDate());
                    tqSpecificationNonStandard.setUpdateBy(SecurityUtils.getStrUserId());
                    tqSpecificationNonStandardService.insertTqSpecificationNonStandard(tqSpecificationNonStandard);
                    // 复制后非标描述ID
                    Long newNonStandardId = tqSpecificationNonStandard.getId();
                    List<TbFile> tbFileList = tbFileService.list(Wrappers.<TbFile>lambdaQuery()
                        .eq(TbFile::getBusinessType, FileBuzType.NON_STANDARD_DESC.getType())
                        .eq(TbFile::getDelFlag, DelFlagEnum.NORMAL.getValue())
                        .eq(TbFile::getBusinessId, oldNonStandardId)
                    );
                    if (CollUtil.isNotEmpty(tbFileList)) {
                        tbFileList.forEach(item -> {
                            item.setId(null);
                            item.setBusinessId(newNonStandardId);
                        });
                        tbFileService.saveBatch(tbFileList);
                    }
                }
            }
        }
    }

    public List<CivilQuotationVO> listByProjectId(Long projectId) {
        List<CivilQuotationVO> civilQuotationList = new ArrayList<>();
        List<QuotationVO> quotationVOS = baseMapper.listByProjectId(projectId);
        if (CollUtil.isNotEmpty(quotationVOS)) {
            String projectName = quotationVOS.get(0).getProjectName();
            Map<String, List<QuotationVO>> collect = quotationVOS.stream()
                .collect(Collectors.groupingBy(QuotationVO::getQuotationNo));
            for (Map.Entry<String, List<QuotationVO>> entry : collect.entrySet()) {
                List<Long> versionList = entry.getValue().stream().map(QuotationVO::getVersion)
                    .collect(Collectors.toList());
                civilQuotationList.add(new CivilQuotationVO(projectName, entry.getKey(), versionList));
            }
        }
        civilQuotationList.sort(Comparator.comparing(CivilQuotationVO::getQuotationNo));
        Collections.reverse(civilQuotationList);
        return civilQuotationList;
    }

    @Override
    public QuotationPriceVO getPriceInfo(Long quotationId) {
        QuotationPriceVO quotationPriceVO = new QuotationPriceVO();
        TqQuotation tqQuotation = this.getById(quotationId);
        // 基准价格合计
        quotationPriceVO.setBasicPrice(tqQuotation.getBasicPrice());
        quotationPriceVO.setBasicPriceFormat(formatter.format(tqQuotation.getBasicPrice()));
        // 选配价格合计
        quotationPriceVO.setOptionalPrice(tqQuotation.getOptionalPrice());
        quotationPriceVO.setOptionalPriceFormat(formatter.format(tqQuotation.getOptionalPrice()));
        // 价格合计
        quotationPriceVO.setPriceTotal(tqQuotation.getPrice());
        quotationPriceVO.setPriceTotalFormat(formatter.format(tqQuotation.getPrice()));

        List<TqBargainPrice> list = tqBargainPriceService
            .list(Wrappers.<TqBargainPrice>lambdaQuery()
                .eq(TqBargainPrice::getQuotationId, quotationId).orderByDesc(TqBargainPrice::getId));
        if (CollUtil.isNotEmpty(list)) {
            TqBargainPrice tqBargainPrice = list.get(0);
            quotationPriceVO.setBargainId(tqBargainPrice.getId());
            // 希望价格
            quotationPriceVO.setExpectedPrice(tqBargainPrice.getBargainPrice());
            quotationPriceVO.setExpectedPriceFormat(formatter.format(tqBargainPrice.getBargainPrice()));
            // 折率（希望价格）
            BigDecimal rate = quotationPriceVO.getExpectedPrice()
                .divide(quotationPriceVO.getPriceTotal(), 4, RoundingMode.HALF_UP);
            String expectedRateStr = rate
                .multiply(new BigDecimal(ONE_HUNDRED))
                .setScale(4, RoundingMode.DOWN)
                .toPlainString();
            quotationPriceVO.setExpectedRate(expectedRateStr);
            // 基价折率（希望价格）
            BigDecimal basicRate = (quotationPriceVO.getExpectedPrice().subtract(quotationPriceVO.getOptionalPrice()))
                .divide(tqQuotation.getOriginalBasicPrice(), 8, RoundingMode.HALF_UP);
            String basicRateStr = basicRate.multiply(new BigDecimal(ONE_HUNDRED))
                .setScale(4, RoundingMode.UP)
                .toPlainString();
            quotationPriceVO.setExpectedBasicRate(basicRateStr);
        } else {
            quotationPriceVO.setExpectedRate(ONE_HUNDRED);
            quotationPriceVO.setExpectedBasicRate(ONE_HUNDRED);
            quotationPriceVO.setExpectedPrice(quotationPriceVO.getPriceTotal());
            quotationPriceVO.setExpectedPriceFormat(formatter.format(quotationPriceVO.getPriceTotal()));
        }
        return quotationPriceVO;
    }

    @Override
    public TbSalesDiscount getSalesDiscountByQuotationId(Long quotationId) {
        // 获取当前审批节点信息
        Map<Long, Flow> flowMap = flowService.getFlowMap(Collections.singletonList(quotationId),
            Arrays.asList(QUOTATION, BARGAIN));
        // 获取当前审批节点信息
        Flow flow = flowMap.get(quotationId);
        if (Objects.nonNull(flow)) {
            NodeRoleEnum nodeRoleEnum = NodeRoleEnum.fromAuditStatusEnum(flow.getStatus());
            return quoteApprovalService.getRoleDiscount(nodeRoleEnum.getRoleId());
        }
        log.info("报价 [{}] 获取审批节点失败", quotationId);
        throw new ServiceException(MessageUtils.message("quotation.flow.undefined"));
    }

    @Override
    public TbSalesDiscount getSalesDiscountByFlow(Integer flowStatus) {
        if (Objects.isNull(flowStatus)) {
            throw new ServiceException(MessageUtils.message("quotation.flow.undefined"));
        }
        NodeRoleEnum nodeRoleEnum = NodeRoleEnum.fromAuditStatusEnum(flowStatus);
        return quoteApprovalService.getRoleDiscount(nodeRoleEnum.getRoleId());
    }

    @Override
    public PageInfo<QuotationApprovalVO> listApproval() {
        List<QuotationApprovalVO> quotationApprovalVOList = this.baseMapper
            .listPendingApproval(SecurityUtils.getStrUserId());
        PageInfo<QuotationApprovalVO> quotationApprovalVOPageInfo = new PageInfo<>(quotationApprovalVOList);
        quotationApprovalVOList = quotationApprovalVOPageInfo.getList().stream().peek(item -> {
            if (Objects.nonNull(item.getPrice())) {
                item.setPriceStr(formatter.format(item.getPrice().setScale(0, RoundingMode.DOWN)));
            }
            if (Objects.nonNull(item.getBargainPrice())) {
                item.setBargainPriceStr(formatter.format(item.getBargainPrice().setScale(0, RoundingMode.DOWN)));
            }
        }).collect(Collectors.toList());
        quotationApprovalVOPageInfo.setList(quotationApprovalVOList);
        return quotationApprovalVOPageInfo;
    }

    @Override
    public H5QuotationDetailVO detailApproval(Long id) {
        H5QuotationDetailVO quotationDetailVO = this.baseMapper.detailApproval(id);
        QuotationPriceVO priceInfo = this.getPriceInfo(id);
        List<H5ElevatorVO> elevatorList = tqSpecificationMapper.listSpecification(id, priceInfo.getBargainId());
        quotationDetailVO.setBargainId(priceInfo.getBargainId());
        // 接受价格
        BigDecimal acceptPrice = BigDecimal.ZERO;
        // 折前基价
        BigDecimal beforeBasicPrice = BigDecimal.ZERO;
        // 折后基价
        BigDecimal afterBasicPrice = BigDecimal.ZERO;
        for (H5ElevatorVO item : elevatorList) {
            item.setPriceStr(formatter.format(item.getPrice().setScale(0, RoundingMode.DOWN)));
            // 设置默认折扣率
            if (Objects.isNull(item.getDiscountRate())) {
                item.setDiscountRate(new BigDecimal(100));
            }
            beforeBasicPrice = beforeBasicPrice.add(item.getBasicPrice());
            BigDecimal discountRate = item.getDiscountRate().divide(new BigDecimal(100), 2, RoundingMode.HALF_UP);
            BigDecimal discountBasicPrice = discountRate.multiply(item.getBasicPrice());
            afterBasicPrice = afterBasicPrice.add(discountBasicPrice);
            item.setDiscountPrice(item.getOptionalPrice().add(discountBasicPrice));
            acceptPrice = acceptPrice.add(item.getDiscountPrice().multiply(new BigDecimal(item.getElevatorNumber())));
        }
        BigDecimal discountRate = (afterBasicPrice.divide(beforeBasicPrice, 6, RoundingMode.DOWN))
            .multiply(new BigDecimal(100)).setScale(0, RoundingMode.DOWN);
        quotationDetailVO.setDiscountRate(discountRate);
        quotationDetailVO.setElevatorList(elevatorList);
        // 基准价格合计
        if (Objects.nonNull(priceInfo.getBasicPrice())) {
            quotationDetailVO.setBasicPriceTotal(priceInfo.getBasicPrice());
            quotationDetailVO.setBasicPriceTotalStr(
                formatter.format(priceInfo.getBasicPrice().setScale(0, RoundingMode.DOWN)));
        }
        // 选配价格合计
        if (Objects.nonNull(priceInfo.getOptionalPrice())) {
            quotationDetailVO.setOptionalPriceTotal(priceInfo.getOptionalPrice());
            quotationDetailVO.setOptionalPriceTotalStr(
                formatter.format(priceInfo.getOptionalPrice().setScale(0, RoundingMode.DOWN)));
        }
        // 价格合计
        if (Objects.nonNull(priceInfo.getPriceTotal())) {
            quotationDetailVO.setPriceTotal(priceInfo.getPriceTotal());
            quotationDetailVO.setPriceTotalStr(
                formatter.format(priceInfo.getPriceTotal().setScale(0, RoundingMode.DOWN)));
        }
        // 希望价格
        if (Objects.nonNull(priceInfo.getExpectedPrice())) {
            quotationDetailVO.setExpectedPrice(priceInfo.getExpectedPrice());
            quotationDetailVO.setExpectedPriceStr(
                formatter.format(priceInfo.getExpectedPrice().setScale(0, RoundingMode.DOWN)));
        }
        // 基价折率
        quotationDetailVO.setBasicPriceRate(priceInfo.getExpectedBasicRate());
        // 同意价格
        quotationDetailVO.setAcceptPrice(acceptPrice);
        quotationDetailVO.setAcceptPriceStr(formatter.format(acceptPrice.setScale(0, RoundingMode.DOWN)));
        // 销售
        String saler = quotationDetailVO.getSaler();
        String insensitiveSaler = saler;
        if (StringUtils.isNotBlank(saler) && saler.length() >= 2) {
            StringBuilder replacedString = new StringBuilder(saler.substring(0, 1));
            for (int i = 1; i < saler.length(); i++) {
                replacedString.append("*");
            }
            insensitiveSaler = replacedString.toString();
        }
        quotationDetailVO.setInsensitiveSaler(insensitiveSaler);
        return quotationDetailVO;
    }
}
